from pandas.core import frame
from torch.utils.data import Dataset
import pandas as pd
from PIL import Image
from pathlib import Path
from .labelmap import LabelMap


class VDataset(Dataset):
    """
    VDataset

    This class is a subclass of the PyTorch Dataset class. It is used to load the videos from the dataset.

    Parameters
    ----------
    csv_file : str
        The path to the csv file containing the dataset.
    root_dir : str
        The path to the root directory of the dataset.
    file_format : str, optional
        The format of the video files. The default is "jpg".
    id_col_name : str, optional
        The name of the column containing the video ids. The default is "video_id".
    label_col_name : str, optional
        The name of the column containing the labels. The default is "label".
    frames_limit_mode : str, optional
        The mode to limit the number of frames. Can be "manual", "csv" or None. The default is {"start" : 0, "end": None}. Ex: To cut 5 from start and 5 from end, use {"start" : 5, "end": -5}
    frames_limit : dict, optional
        The number of frames to limit the dataset to. The default is 1.
    frames_limit_col_name : str, optional
        The name of the column containing the number of frames. The default is "frames".
    video_transforms : object, optional
        The transformations to apply to the loaded videos. The default is None. (Refere https://github.com/hassony2/torch_videovision)
    label_map : LabelMap, optional
        The label map to use. The default is None. (refere https://github.com/nzx9/vdataset)

    Returns
    -------
    VDataset
        The VDataset object.
    """

    def __init__(self, csv_file: str, root_dir: str, file_format: str = "jpg", id_col_name: str = "video_id", label_col_name: str = "label", frames_limit_mode: str = None, frames_limit: dict = {"start" : 0, "end": None}, frames_limit_col_name: str = "frames", video_transforms=None, label_map: LabelMap = None):
        self.dataframe = pd.read_csv(csv_file)
        self.root_dir = root_dir
        self.file_format = '*{}'.format(
            file_format) if file_format[0] == "." else '*.{}'.format(file_format)
        self.id_col_name = id_col_name
        self.label_col_name = label_col_name
        self.frames_limit_mode = frames_limit_mode
        self.frames_limit_col_name = frames_limit_col_name
        self.transform = video_transforms
        self.label_map = label_map

        
        self.frames_limit = {"start": 0, "end": 0}
        self.frames_limit["start"] = 0 if frames_limit["start"] == None else frames_limit["start"]
        self.frames_limit["end"] = 0 if frames_limit["end"] == None else frames_limit["end"]

    def __getitem__(self, index):
        row = self.dataframe.iloc[index]
        label_name = row[self.label_col_name]

        if self.label_map != None:
            label = self.label_map.to_id(label_name)
            if label == None:
                raise ValueError("label can't be None")
        else:
            label = label_name
        video_id = row[self.id_col_name]

        if self.frames_limit_mode == "csv":
            self.frames_limit = row[self.frames_limit_col_name]

        frames_list = list(
            Path('{}/{}'.format(self.root_dir, video_id)).glob(self.file_format))

        if len(frames_list) == 0:
            raise FileNotFoundError('Error: No frames found.')
        elif (self.frames_limit_mode == "manual" or self.frames_limit_mode == "csv"):
            if (self.frames_limit["end"] == 0):
                self.frames_limit["end"] = len(frames_list)
            frames_list = frames_list[self.frames_limit["start"]: self.frames_limit["end"]]

        frames = [Image.open(f).convert('RGB') for f in frames_list]

        if self.transform:
            frames = self.transform(frames)
        return frames, label

    def __len__(self) -> int:
        return len(self.dataframe)



import sys

sys.path.append( 'C:\\CODE\\vdataset' )

from vdataset.vdataset import VDataset
from vdataset.labelmap import LabelMap
import torch
from torchvideotransforms import video_transforms, volume_transforms

# DEVICE SELECTING
DEVICE = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
print("Executing on: {}".format(DEVICE))

# CONSTANTS GOES HERE
FRAME_SIZE = (224, 224)

DATASET_ROOT = "C:\\CODE\\GI-RESNET3D\\data\\20BN-JESTER"
CSV_PATH = {
    "train": "{}/Train.csv".format(DATASET_ROOT),
    "validation": "{}/Test.csv".format(DATASET_ROOT)
    }

DATA_DIR = {
    "train": "{}/Train".format(DATASET_ROOT),
    "validation" : "{}/Validation".format(DATASET_ROOT)
    }


labelMap = LabelMap(labels_csv=CSV_PATH["train"], labels_col_name='label')
labelMap.print()

transforms = {
    "train": video_transforms.Compose([video_transforms.RandomRotation(30), 
              video_transforms.Resize(FRAME_SIZE), 
              volume_transforms.ClipToTensor()]),
    "validation": video_transforms.Compose([video_transforms.Resize(FRAME_SIZE), 
                                            volume_transforms.ClipToTensor()])
}


datasets = {
    "train": VDataset(csv_file=CSV_PATH["train"], root_dir=DATA_DIR['train'], video_transforms=transforms["train"], label_map=labelMap, frames_limit_mode="manual", frames_limit={"start": 5, "end": -5},),
    "validation": VDataset(csv_file=CSV_PATH["validation"], root_dir=DATA_DIR['validation'], video_transforms=transforms["validation"], label_map=labelMap),
}

dataloaders = {
    "train": torch.utils.data.DataLoader(datasets["train"], batch_size=16, shuffle=False, pin_memory=True),
    "validation": torch.utils.data.DataLoader(datasets["validation"], batch_size=16, shuffle=False, pin_memory=True)
}
print(dataloaders)


for i, data in enumerate(dataloaders['train']):
    print("Batch : {} ".format(i))
    imgs, labels = data
    print(imgs.size())
    print(labels)
    break;



import pandas as pd


class LabelMap():
    """
    LabelMap

    This class is a map between labels and ids.

    Parameters
    ----------
    labels_csv : str
        The path to the csv file containing the labels and ids.
    labels_col_name : str
        The name of the column containing the labels.
    ids_col_name : str
        The name of the column containing the ids.
    id_type : type
        The type of the ids.

    Attributes
    ----------
    label_map : dict
        The map between labels and ids.
    label_map_inv : dict
        The map between ids and labels.
    label_count : int
        The number of labels.
    label_data : pandas.DataFrame
        The dataframe containing the labels and ids.

    Methods
    -------
    add(label)
        Add a label to the label map.
    force_get(label)
        Get the id of a label or if id not found then add the label and get.
    to_text(id)
        Get the label of an id.
    to_id(label)
        Get the id of a label.
    map()
        Get the label map as dict.
    labels()
        Get the list of labels.
    print()
        Print the label map as table.
    count()
        Get the number of labels.
    get_csv()
        Get the data in the provided csv file.

    """
    __label_map__ = {}
    __label_map_inv__ = None
    __label_count__ = 0
    __label_data__ = None
    __cache_outdated__ = False

    def __init__(self, labels_csv=None, labels_col_name=None, ids_col_name=None, id_type=int):
        if labels_csv != None:
            if id_type != float and id_type != int:
                raise ValueError("id_type must be a int or float type")
            self.__type__ = id_type
            self.__label_data__ = pd.read_csv(labels_csv)
            self.__from_csv__(self.__label_data__,
                              labels_col_name, ids_col_name)

    def __from_csv__(self, label_data, labels_col_name, ids_col_name):
        self.__cache_outdated__ = True
        if labels_col_name == None:
            raise ValueError(
                "lable_col_name can't be None if labels_csv is not None")
        for i in range(0, len(label_data)):
            row = label_data.iloc[i]
            if self.__label_map__.get(row[labels_col_name]) == None:
                _id = self.__type__(self.__label_count__)
                if ids_col_name != None:
                    _id = row[ids_col_name]
                self.__label_map__[row[labels_col_name]] = _id
                self.__label_count__ += 1

    def add(self, label):
        """
        Add a label to the label map.

        Args
        ----------
        label -> string
        """
        if self.__label_map__.get(label) == None:
            self.__label_map__[label] = self.__type__(self.__label_count__)
            self.__cache_outdated__ = True
            self.__label_count__ += 1

    def force_get(self, label) -> int:
        """
        Get the id of the given label, if id of the label is None then add the label and return id.

        Args
        ----------
        label -> string
        """
        if self.__label_map__.get(label) == None:
            self.add(label)
        return self.__label_map__.get(label)

    def to_text(self, id) -> str:
        """
        Get the label of an id.

        Args
        ----------
        id -> int/ float
        """
        if(self.__label_map_inv__ == None or self.__cache_outdated__):
            self.__label_map_inv__ = {v: k for k,
                                      v in self.__label_map__.items()}
            self.__cache_outdated__ = False
        return self.__label_map_inv__[id]

    def to_id(self, label) -> int:
        """
        Get the id of the given label.

        Args
        ----------
        label -> string
        """
        return self.__label_map__[label]

    def map(self) -> dict:
        """
        Get the label map as dict.

        Args
        ----------
        None
        """
        return self.__label_map__

    def labels(self) -> list:
        """
        Get the list of lables

        Args
        ----------
        None
        """
        lst = []
        for label in self.__label_map__:
            lst.append(label)
        return lst

    def print(self) -> None:
        """
        Print the label map as table

        Args
        ----------
        None
        """
        print("{:>7} | {:<50}".format("ID", "LABEL"))
        print("{:>7}+{:<50}".format("--------",
              "----------------------------------------------"))
        for label in self.__label_map__:
            print("{:>7} | {:<50}".format(self.to_id(label), label))

    def count(self) -> int:
        """
        Get the number of labels.

        Args
        ----------
        None
        """
        return len(self.__label_map__)

    def get_csv(self) -> pd.DataFrame:
        """
        Get the data in the provided csv file.

        Args
        ----------
        None
        """
        return self.__label_data__
